"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresPlaceholderResolver = void 0;
const core_1 = require("./core");
const error_1 = require("./error");
/**
 * Get name of i'th positional placeholder.
 */
const positionalPlaceholderName = (i) => "$" + (i + 1);
class PostgresPlaceholderResolver {
    constructor(placeholders) {
        this.placeholders = placeholders;
        this.memo = new Map();
        this.values = [];
    }
    usedNames() {
        return [...this.memo.keys()];
    }
    allocScalar(value) {
        const i = this.values.length;
        this.values.push(value);
        return positionalPlaceholderName(i);
    }
    allocArray(values) {
        if (values.length === 0) {
            return core_1.EMPTY_LIST_SQL;
        }
        const offset = this.values.length;
        this.values.push(...values);
        // Build ($1, $2, ...).
        const text = values.map((_, i) => positionalPlaceholderName(offset + i)).join(", ");
        return `(${text})`;
    }
    resolvePlaceholder(name) {
        let text = this.memo.get(name);
        if (text !== undefined) {
            return text;
        }
        const value = this.placeholders[name];
        if (value instanceof Array) {
            text = this.allocArray(value);
        }
        else {
            text = this.allocScalar(value);
        }
        this.memo.set(name, text);
        return text;
    }
    static resolve(statement, placeholders) {
        const resolver = new PostgresPlaceholderResolver(placeholders);
        const replacedStatement = statement.replace(core_1.PLACEHOLDER_REGEXP, (_text, name) => resolver.resolvePlaceholder(name));
        const err = error_1.SqlPlaceholderError.checkExhaustivity(placeholders, resolver.usedNames());
        if (err != null) {
            throw err;
        }
        return [replacedStatement, resolver.values];
    }
}
exports.PostgresPlaceholderResolver = PostgresPlaceholderResolver;
//# sourceMappingURL=resolver_postgres.js.map